package Lstatobj;

use strict;
use Carp;
use Misc;

#export module sybmols
require Exporter;
use vars qw ( @ISA @EXPORT @EXPORT_OK );
@ISA = qw(Exporter);

@EXPORT = qw( LoadObjectByName LoadObjects LoadObjectByFileName );
#@EXPORT_OK = qw( %ObjectTypes  );



BEGIN {
#register all known graphs

 RegisterClass ('TCPUGrapher', '','cpu','CPU_DESC',
 sub  { my $obj;  $obj = TCPUGrapher -> new ();  return ($obj); } );
 RegisterClass ('TMEMGrapher', '','mem','MEM_DESC',
 sub  { my $obj;  $obj = TMEMGrapher -> new ();  return ($obj); } );
 RegisterClass ('TLOADGrapher', '','load','LOAD_DESC',
 sub  { my $obj;  $obj = TLOADGrapher -> new ();  return ($obj); } );
 RegisterClass ('TPARTGrapher', '','part','PART_DESC',
 sub  { my $obj;  $obj = TPARTGrapher -> new ();  return ($obj); } );
 RegisterClass ('TDISKGrapher', '','disk','DISK_DESC',
 sub  { my $obj;  $obj = TDISKGrapher -> new ();  return ($obj); } );
 RegisterClass ('TINTGrapher', '','int','INT_DESC',
 sub  { my $obj;  $obj = TINTGrapher -> new ();  return ($obj); } );
 RegisterClass ('TIPGrapher', '','ip','IP_DESC',
 sub  { my $obj;  $obj = TIPGrapher -> new ();  return ($obj); } );
 RegisterClass ('TPINGGrapher', '','ping','PING_DESC',
 sub  { my $obj;  $obj = TPINGGrapher -> new ();  return ($obj); } );

}




#load all objects in memory
sub LoadObjects {
my $objref = shift;

my $objdir = $config {'OBJ_DIR'};
opendir (DIR, $objdir) || carp "Can't opendir $objdir: $! \n";
my @objfiles = grep { /^[a-z]\w+\.obj/i } readdir (DIR);
closedir DIR;

foreach my $filename (@objfiles) {
 push (@$objref, LoadObjectByFileName ($objdir.$filename));
}
}

#load one object by name
sub LoadObjectByName  {
my $name = shift;
return LoadObjectByFileName  (  $config {'OBJ_DIR'}.$name.'.obj');
}

#load one object from file
sub LoadObjectByFileName  {
my $filename = shift;
my $MyConfig = \%config;
 open (FILE, "<$filename" ) || croak "Can't open object file $filename: $! \n";
 my $objtype = <FILE>; #Object's type i first line
 chomp ($objtype);
 close FILE;
 my $obj = CreateObjByClass ($objtype);
 if (! defined ($obj)) {  carp "Unknown object type $objtype in $filename .\n" }

$obj ->LoadFromFile ($filename); #loading data of object
return $obj;
}

#-----------------------------------------------------------------------------------------------
package TGrapher;
use CGI qw (:standard);
use CGI::Carp qw (fatalsToBrowser croak );
use RRDs;
use POSIX qw(strftime mktime ctime);
use Misc;
use Misc qw ( %config );

my $Debugging = 0;

sub new {
 my $proto = shift;
 my $class = ref ($proto) || $proto;
 my $self = {};
 #setting default values
 my $configref = \%config;
 my %desc = ();
 $self -> {'CONFIG'} = $configref; #reference to program's config
 $self -> {'HAS_INFO'} = 0; #set if object has additional info
 $self -> {'HELP_FILE'} = 'generic.html'; #set default help file
 $desc  {'NAME'} = 'default';
 $desc  {'TITLE'} = "\@\@TITLE\@\@";
 $desc  {'UPDATE_PERIOD'} = $$configref { 'DEF_UPDATE_PERIOD' };
 $desc  {'WIDTH'} = $$configref { 'DEF_WIDTH' };
 $desc  {'HEIGHT'} = $$configref { 'DEF_HEIGHT' };
 my @Checkboxes; #table for name of checkboxes
 my @Radioboxes; #table for name of radioboxes
 $self ->{'CHKBOX'} = [ @Checkboxes ];
 $self ->{'RADIOBOX'} = [ @Radioboxes ];
 $self -> {'DESC'} = {%desc};

 bless ($self, $class);
 return $self;
}

#add to Checkbox table (for editing)
sub AddToCHKBOX {
 my $self = shift;
 my $box = $self ->{'CHKBOX'};
 foreach  my $val ( @_) {push (@$box, $val);}
}
#add to Radiobox table (for editing)
sub AddToRADIOBOX {
 my $self = shift;
 my $box = $self ->{'RADIOBOX'};
 foreach  my $val ( @_) {push (@$box, $val);}
}

#for debugging
sub debug {
 my $class = shift;
 $Debugging = shift;
}


#perparing create RRDfile, common for all objects
sub Create_Object {
 my $self = shift;
 my $rrdstep = shift;
 my $Refsource = shift;
 #empty
 return 1;
}

sub CreateRRD {
 my $self = shift;
 my $rrdstep = $self -> GetUpdatePeriodInSec ();

#primary struct rrdfile
my @source =("--step",$rrdstep );

 my $ok =0;
 #data for each object
 if ($self -> Create_Object ($rrdstep * 2, \@source) ) { #value for update data sources
 #test if RRA record need
 my $norra=1;
 foreach (@source) {
  if ($_ =~ /^RRA/ ) {
   $norra = 0;
   last;
  }
 }

 if ($norra) {
  #add default rra records
 push (@source, "RRA:AVERAGE:0.5:1:600");
 push (@source, "RRA:AVERAGE:0.5:6:700");
 push (@source, "RRA:AVERAGE:0.5:24:775");
 push (@source, "RRA:AVERAGE:0.5:288:797");
 }
 my $rrdfilename = $self -> {'RRD_FILENAME'}; #name file to create

 RRDs::create ($rrdfilename, @source);
 my $ERROR = RRDs::error;
# carp "$0: unable to create `$rrdfilename': $ERROR\n" if $ERROR;
 $ok =1 if (! $ERROR);
 }
 return ($ok);
}


sub Update_Object {
 #empty
 return ("");
}

#update RRDfile
sub UpdateRRD {
 my $self = shift;

 my $values = $self -> Update_Object ();
 my $rrdfilename = $self -> {'RRD_FILENAME'}; #name file to update
 my $ok=0;
 if ($Debugging >  1 )  {
  print "Update $rrdfilename: \n$values\n";
 }
 if ($values) {
 RRDs::update ($rrdfilename, $values);
 my $ERROR = RRDs::error;
# carp "$0: unable to update `$rrdfilename': $ERROR\n" if $ERROR;
 $ok =1 if (! $ERROR);
 }
return ($ok);
}

#test security of object - true: object OK
sub Check_Security {
 return (1);
}


#convert PERIOD to PERIOD in sec
sub GetUpdatePeriodInSec {
 my $self = shift;
 $_ = $self -> {'DESC'}{'UPDATE_PERIOD'};
 (my $first, my $second) =  /(\d+)([mhdwM]?)/;
 if ($second) {
  #change to second
  SWITCH: {
   $second eq  'm' && do {
   		           $first *= 60;
   		           last SWITCH;	
   		           };
   $second eq  'h' && do {
   		           $first *= 3600;
   		           last SWITCH;
   		           };
   $second eq  'd' && do {
   		           $first *= 3600 * 24;
   		           last SWITCH;
   		           };
   $second eq  'w' && do {
   		           $first *= 3600 * 24 * 7;
   		           last SWITCH;
   		           };
   $second eq  'M' && do {
   		           $first *= 3600 * 24 * 28;
   		           last SWITCH;
   		           };
  carp ("Unknown UPDATE PERIOD in $self->{'DESC'}{'NAME'}\n");
  }
 }
 return $first;
}

#set or read object's name
sub name {
 my $self = shift;
 if (@_) {
  my $newname = shift;
  croak "Invalid object name: $newname \n" if (!($newname =~ /^\w+$/));
  $self -> {'DESC'}{'NAME'} = $newname;
  $self -> {'RRD_FILENAME'} = $self -> {'CONFIG'}{'RRD_DIR'}.$newname.'.rrd'; # new rrd filename'';
  $self -> {'PNG_FILENAME'} = $self -> {'CONFIG'}{'STAT_PNG_DIR'}.$newname; # new png graph filename'';
  $self -> {'OBJ_FILENAME'} = $self -> {'CONFIG'}{'OBJ_DIR'}.$newname.'.obj'; # new object filename'';
  $self -> {'PNGURL'} = $self -> {'CONFIG'}{'STAT_PNG_URL'}.$newname; # new object filename'';
 }
 return $self -> {'DESC'}{'NAME'};
}

#set or read object's update period
sub UpdatePeriod {
 my $self = shift;
 if (@_) {$self -> {'DESC'}{'UPDATE_PERIOD'} = shift };
 return $self -> {'DESC'}{'UPDATE_PERIOD'};
}

#save object's data to default file
sub Save {
 my $self = shift;
 my $filename = $self ->{'OBJ_FILENAME'}; #filename
 $self -> SaveToFile ($filename);
}

#save object's data to  file
sub SaveToFile {
 my $self = shift;
 my $filename= shift;
 my $descref =  $self -> {'DESC'};
 #save content DESC to file
 open (FILE, ">$filename" ) || carp "Can't create file $filename: $! \n";
 $self =~ /(\w+)=.*/; #extract object type
 print FILE "$1\n"; #Save object type
 foreach my $key  (keys %$descref) {
 my $value = $descref->{$key};
 if ( ref ($value) ){
  #save reference
  if  (ref ($value) eq "ARRAY") {
   #key as array
  print FILE "\[$key\]\n"; #begin
#  print @$value;
  if (@$value)  { #if has any elements
  for (my $i=0;$i<@$value;$i++) {
   my $HashRef = $$value[$i];
   foreach my $key2 (keys %$HashRef) {
    print FILE "$key2$i=",$$HashRef{$key2},"\n"; #print each value with number of position
   }
  }
  }
  print FILE "\[/$key\]\n"; #end
 }
 } else {
  #simply value
  print FILE "$key=$value\n";
  }
 }
 close FILE;
}

#load object's data from file
sub LoadFromFile {
 my $self = shift;
 my $filename = shift;
 my $descref =  ($self -> {'DESC'});

 #load content DESC to file
 open (FILE, "<$filename" ) || carp "Can't open file $filename: $! \n";
 my @content=<FILE>;
 close FILE;
 #ignore object type
 shift @content;
 if (@content) {
 my $i=0;
 while ($i < @content)  {
  chomp (my $line=$content[$i]);
  if ($line =~ /=/ ) {
  #simply
  (my $key, my $value ) = split (/\=/,$line,2);
  $descref->{$key} = $value;
  $i++;
  }elsif ($line =~ /^\[(\w+)\]/ ) {
  #begin of array
  my @NewArray;
  my $ArrayName =$1;
  my $lastnum = -1;
  my $hashref;
  $i++;
  while ($content [$i++] =~ /^([a-z]+)(\d+)=(.*)/i) {
   #elements of array
   my $key2=$1;
   my $row= $2;
   my $value2=$3;
   chomp ($value2);
   $value2='' if (!defined ($value2));
   if ($row != $lastnum ) {
     #new row of data
     my %rowhash;
     $hashref = { %rowhash };
     push (@NewArray, $hashref);
     $lastnum ++;
    }
   $hashref-> {$key2} = $value2;
  }
 $descref->{$ArrayName} = [ @NewArray ]; #add to object
 } else {
  $i++; #skip end of array or ignore this line
  }
 } #end while
 }
 #setting some variables
 $self -> name ($self -> {'DESC'}{'NAME'}); # set new name for rrd, png;

}

#load object's data from default file
sub Load {
 my $self = shift;
 my $filename = $self->{'OBJ_FILENAME'}; #filename
 LoadFromFile ($self, $filename);
}


#prepare rrd draw source for each object
sub Draw_Object {
 my $self = shift;
 my $rrd =shift;
 my $Refsource =shift;
 #empty
}


sub Draw {
 my $self = shift;
 my $from =shift;
 my $to =shift;
 my $options = shift; #additional options for draw
 my $objoptions = shift; #additional options for object

 if (! defined ($options)) {
  $options ='';
 }
 if (! defined ($objoptions)) {
  $objoptions ='';
 }
 if ($objoptions) { #process changes in object's property
  chomp ($objoptions);
  $objoptions =~ s/^[^\w]+//; #clear unwanted chars from beginning
  my @addopt = split (/\&/, $objoptions);
  foreach (@addopt) {
   my ($key, $value) = split (/\=/,$_,2);
   $self->{'DESC'}{$key} = $value if (defined ($self->{'DESC'}{$key}));
  }
 $instance ++;
 my $tmp_hash=time; #get current time for tempolaty name
 $tmp_hash .= '_'.$$; #add PID to temp name
 $self -> {'PNG_FILENAME'} .='_'.$instance.'_'.$tmp_hash;
 $self -> {'PNGURL'} .='_'.$instance.'_'.$tmp_hash;
 croak "Security vioatlion" if (! ( $self ->Check_Security ())); #check security
 }

 my @source = ();
 #some common values
 my $rrd = $self->{'RRD_FILENAME'};

 push (@source,"--title");
 my $title=$self -> {'DESC'}{'TITLE'};
 $title =~ s/\@\@TITLE\@\@/$dict{$self -> {'TITLE_TAG'}}/g; #set default title if defined
 push (@source,$title);
 push (@source,"--start");
 if ($from =~ /[^\d\-]/) {
  #period in short form
  $from = '-'.$from;
 }
 push (@source,"$from");
 push (@source,"--end");
 push (@source, $to);

 #save from and to values
 $self -> {'FROM'} = $from;
 $self -> {'TO'} = $to;

# push (@source,"--interlace");
 push (@source,"--imgformat");
 push (@source,"PNG");
 my $width=$self->{'DESC'}{'WIDTH'};
 my $height=$self->{'DESC'}{'HEIGHT'};
 $width *= $self -> {'CONFIG'}{'H_SCALE'} if (defined ($self -> {'CONFIG'}{'H_SCALE'}));
 $height *= $self -> {'CONFIG'}{'V_SCALE'} if (defined ($self -> {'CONFIG'}{'V_SCALE'}));
 push (@source,"--width=$width");
 push (@source,"--height=$height");
 push (@source,"--lower");
 push (@source,"0");
 push (@source,"--lazy");
 if  ($from =~/m/) {
 #different scale form month
 push (@source,"--x-grid");
 push (@source,"HOUR:24:DAY:7:DAY:2:0:\%d-\%m");
 }
 if  ($from =~/^\-\d+$/) {
 #for very short periods
 push (@source,"--x-grid");
 push (@source,"SECOND:20:MINUTE:1:MINUTE:2:0:\%H:\%M");
 }

 #connect with object data
 $self ->Draw_Object ($rrd, \@source);

 my $pngname = $self-> {'PNG_FILENAME'};

 #add from suffix
 my $from2 = $from;
 $from2 =~ s/^\-//;
 $from2 = "_$from2\.png";
 $pngname .= $from2;
 my $pngurl = $self->{'PNGURL'}.$from2;

(my $graphret, $width, $height) = RRDs::graph $pngname, @source;
my $ERROR = RRDs::error;
croak "$0: unable to graph $pngname: $ERROR\n" if $ERROR;

if (! ($options =~ /nohtml|png|only_name/i) ) {
#set here flag
print "<A name=\"$self->{DESC}{NAME}\"></A>\n";

if (! ($options =~ /nomenu/) ) {
#show additional functions
my $surl = $self -> {'CONFIG'}{'SCRIPT_URL'};
my $selfurl = $self -> {'CONFIG'}{'SELF_URL'};
my $iurl = $self -> {'CONFIG'}{'ICONS'};
my $editurl = $self -> {'CONFIG'}{'EDIT_URL'};
my $docurl = $self -> {'CONFIG'}{'DOC'}.$self -> {'HELP_FILE'};
my $live_locked = $self -> {'CONFIG'}{'LIVE_LOCKED' };
my $moreinfo = $self -> {'CONFIG'}{'SHOW_MORE_INFO' };


my $objname = $self->{'DESC'}{'NAME'};
LoadLangData ('','draw');

print "<BR><TABLE  BORDER=0 WIDTH=$width BGCOLOR=$dict{'TABLE_HEADER_COLOR'} CELLSPACING=0><TR><TD VALIGN=CENTER>\n";

print "<A target=_blank HREF=$surl?obj=$objname\&template=table.html\&from=";
print $self->{'FROM'},'&to=',$self->{'TO'},"><IMG SRC=$iurl","table.gif BORDER=0 ALT=\"$dict{DRAW_TABLE}\"></A> \n";
if ($self -> {'HAS_INFO'} and $moreinfo) { #print icon additional info
 print "<A target=_blank HREF=$surl?obj=$objname\&action=info><IMG SRC=$iurl","info.gif BORDER=0 ALT=\"$dict{DRAW_INFO}\"></A> \n";
}
my $height2 = $height + 50;
my $width2 = $width + 20;

if (! $live_locked ){
print "<A HREF=\"javascript: void window.open(\'$surl?obj=$objname\&action=live\','',\'left=100,top=100,width=$width2,height=$height2\')\">\n";
print "<IMG SRC=$iurl","live.gif BORDER=0 ALT=\"$dict{DRAW_LIVE}\"></A> \n";
}
print "<A target=_blank HREF=\"$editurl?obj=$objname\"><IMG SRC=$iurl","prop.gif  BORDER=0 ALT=\"$dict{DRAW_PROP}\"></A> \n";

print "<A HREF=\"javascript: void refresh_it (\'$selfurl\#$self->{DESC}{NAME}\') \"><IMG SRC=$iurl","refone.gif  BORDER=0 ALT=\"$dict{DRAW_REFRESH}\"></A>";
print "</TD><TD ALIGN=RIGHT>\n";
print "<A target=_blank HREF=$docurl#draw><IMG SRC=$iurl","minihelp.gif BORDER=0 ALT=\"$dict{DRAW_HELP}\"></A> \n";
print "</TD></TR></TABLE>\n";
}

#show graph
print "<IMG SRC=$pngurl BORDER=0 WIDTH=$width HEIGHT=$height><BR>\n";
}

if  ($options =~ /png/i)  {
#generate png data
 print header (-type=>'image/png', -expires=>'+5s');
 open (PLIK,"<$pngname") or croak "Cant open $pngname";
 my @content =<PLIK>;
 print @content;
 close PLIK;
} elsif  ($options =~ /only_name/i)  {
 #print png name
 $pngname =~ s/^.*\///;  #remove directory info
 print $pngname;
 }

}

sub ShowData { #print table with data
 my $self = shift;
 my $from = shift;
 my $to = shift;

my $datetimeformat = $self -> {'CONFIG'}{'DATETF'}; #date and time format
my @source=('AVERAGE');
#change to another RRA type
$source [0] = $self -> {'RRATYPE'} if (defined ($self -> {'RRATYPE'}));

push (@source ,'--start');
push (@source ,$from);
push (@source , '--end');
push (@source , $to);

#show table with values

my ($start,$step,$names,$array) = RRDs::fetch ($self->{'RRD_FILENAME'}, @source);
my $ERROR = RRDs::error;
croak "RRD ERROR: $ERROR\n" if $ERROR ;

LoadLangData ('','draw');

print "<TABLE BORDER=1 BORDERCOLOR=#000000 CELLSPACING=0><TR BGCOLOR=#80FF80 ><TD>$dict{DRAW_TIME_SAMPLE}</TD>";
map {printf("<TD>%s</TD>",$_)} @$names ;
print "</TR>\n";
foreach my $line (@$array){
  print "<TR><TD BGCOLOR=yellow >",strftime ($datetimeformat, localtime($start)),"</TD>\n";
  $start += $step;
  foreach my $val (@$line) {
    if (defined ($val) ) {printf "<TD>%12.2f</TD>", $val }
     else {print "<TD>Undef</TD>"}
  }
  print "</TR>\n";
}
print "</TABLE>\n";
}

#EXTRA STATISIC
sub ExtraData {
 my $self = shift;
# my $RefRes = shift; #reference to table 0-Total, 1 AVG, 2 MAX, 3 MIN
 my %ResHash;

 #get from &to from object
 my $from =  $self -> {'FROM'};
 my $to =$self -> {'TO'};

my @source=('AVERAGE');
#change to another RRA type
$source [0] = $self -> {'RRATYPE'} if (defined ($self -> {'RRATYPE'}));

push (@source ,'--start');
push (@source ,$from);
push (@source , '--end');
push (@source , $to);

#show table with values

my ($start,$step,$names,$array) = RRDs::fetch ($self->{'RRD_FILENAME'}, @source);
my $ERROR = RRDs::error;
croak "RRD ERROR: $ERROR\n" if $ERROR ;

#alocate data for statistic
my @stat;
foreach (@$names) {
 push (@stat, [ 0,0,0,1E10,0 ]);   #defined samples, sum, max, min, last
}

foreach my $line (@$array){
  for ( my $i=0; $i<@$line;$i++) {
  my $val =$line->[$i];
    if (defined ($val) ) {
     $stat[$i][0]++; #inc valid samples
     $stat[$i][1] += $val;   #sum
     $stat[$i][2] = $val if ($val > $stat[$i][2]); #max
     $stat[$i][3] = $val if ($val < $stat[$i][3]);  #min
     $stat[$i][4] = $val; #last

      }
  }
}


foreach my $st (@stat) {
 my $AVG = 0;
 $AVG = $st->[1] / $st->[0] if ($st->[0]);
 # my $kas=/"/";
 my $Total= $st->[1] * $step;

 #count results
 $st->[3] = 0  if ( ! $st->[0] ); #all undefined samples, set min to 0
 $ResHash {$names->[0]} = [$Total, $AVG, $st->[2],$st->[3],$st->[4] ];
 shift @$names;

 }
 return \%ResHash;
}


#EDIT

#prepare EDIT Object
sub Edit {
 my $self = shift;
 my $RefTags =shift;
 LoadLangData ('', 'edit'); #load info about edit parameters

#tabele update periods
 my %Upd_Periods = ('s',$dict{'EDIT_SEC'}, 'm',$dict{'EDIT_MIN'}, 'h', $dict{'EDIT_HOUR'}, 'd', $dict{'EDIT_DAY'},
 	'w', $dict{'EDIT_WEEK'}, 'M', $dict{'EDIT_MONTH'});
 my @periods= ('s', 'm', 'h', 'd', 'w', 'M');

 $$RefTags {'TEMP_OBJ_NAME'} = $self->{'DESC'}{'NAME'};
 $$RefTags {'HELP_FILE'} = $self->{'HELP_FILE'};
 my $DescRef = $self->{'DESC'};

 #extract all DESC tags;
 foreach my $key (keys %$DescRef) {
   $$RefTags {$key} = $$DescRef {$key};
 }
 #set update period

 my $period = $DescRef ->{'UPDATE_PERIOD'};
 $period =~ /(\d+)(\w*)/;
 my $value = $1;
 my $unit = $2;
 $unit = 's' if (! $unit); #default for second
 my @unit_list = scrolling_list('period_unit',
                                       \@periods,
                                       $unit,1,'',
                                       \%Upd_Periods);
  $$RefTags {'UPDATE_PERIOD_UNIT'} =  \@unit_list;
  $$RefTags {'UPDATE_PERIOD_VAL'} = $value;
 #prepate checkbox values;
 my $CheckBOX = $self ->{'CHKBOX'};
 if (@$CheckBOX) {
  foreach my $ch (@$CheckBOX) {
   my $newname= $ch.'_BOX';
   if ($DescRef ->{$ch}) { $$RefTags {$newname} = 'CHECKED'}
   else { $$RefTags {$newname} = ''}
   }
 }

 #prepare for radiobox values;
 $CheckBOX = $self ->{'RADIOBOX'};
 if (@$CheckBOX) {
  foreach my $ch (@$CheckBOX) {
   my $newname= $ch.'_BOX';
   my $val = $DescRef ->{$ch};
   for ( my $i=0; $i < 5; $i++) {
    if ($i ==$val) {$$RefTags {$ch.'_'.$i} = 'CHECKED' }
     else {$$RefTags {$ch.'_'.$i} = '' }
   }
   }
 }
 Gen ($RefTags, 'eobjpri1.html', 1);
 #for additional primary options
 $self -> EditObjPrimary ($RefTags); #edit additional
 Gen ($RefTags, 'eobjpri2.html', 1);
 #for advanced options
 $self -> EditObjAdvanced ($RefTags);

 #finish prepare
 Gen ($RefTags, 'eobjend.html', 1);

}


#edit additional primary fields
sub EditObjPrimary {
}

#edit advanced fields
sub EditObjAdvanced {
}


sub PostEdit {
 my $self = shift;
 my $RefTags =shift;
 my $RefPostTags =shift;
 my $displayPage =shift;
 #process posttags
# while ( my ($key, $value) =each  %$RefPostTags) {
 # print "$key=$value\n--";
 #}

 $$RefTags {'OBJ_NAME'} = $$RefPostTags {'OBJ_NAME'};

 my $DescRef = $self->{'DESC'};
 #store all DESC tags;
 foreach my $key (keys %$DescRef) {
  if (defined ($$RefPostTags {$key})) {
   $$DescRef {$key} = $$RefPostTags {$key} if (! ($key =~ /\d$/));
   }
 }
 #convert UPDATE_PERIOD
 my $update_unit = $$RefPostTags {'period_unit'};
 if (defined ($update_unit)) {
  $update_unit ='' if ($update_unit eq 's');
  $DescRef ->{'UPDATE_PERIOD'} = $$RefPostTags {'UPDATE_PERIOD_VAL'}.$update_unit;
 }

 #process checkboxes only if primary
 if (!($$RefPostTags {'RESULT'} =~ /OK_EL|NEW_EL|DEL_EL/)) {
  my $CheckBOX = $self ->{'CHKBOX'};
 if (@$CheckBOX) {
  foreach my $ch (@$CheckBOX) {
   my $newname= $ch.'_BOX';
   if (defined ($$RefPostTags {$newname})) { $DescRef ->{$ch} = 1}
   else { $DescRef ->{$ch} = 0}
   }
 }
 }

 $self -> PostEditObjPrimary ($RefPostTags); #save additional primary options
 $self -> PostEditObjAdvanced ($RefPostTags); #save advanced options
 #save current context
 $self->Save;

 #prepare tags for new display
 $self -> Edit ($RefTags) if ($displayPage);
}

sub PostEditObjPrimary {
}

sub PostEditObjAdvanced {
}

sub EndEdit {
 my $self = shift;
 my $RefTags =shift;
 my $Finally =shift;
 my $Primary = $$RefTags {'OBJ_NAME'}; #name of primary object

 if ($Finally==1) {
  #I'm sure to recreate rrd and end edit
  unlink ($self -> {'OBJ_FILENAME'}); #delete tempolary object file
 #test if DEMO version
 if (IF_DEMO ()) {
  Gen ($RefTags, 'demo.html', 1);
 } else {
  $self -> name ($Primary); #change name to primary
  my $err;
  if ($self->Check_Security ()) {
  $self -> Save; #save with new values
  if (! ($err = Misc::SendMsg (3, $Primary))) { #send message to recreate rrd file
  Gen ($RefTags, 'thanks.html', 1);
  } else {
    #problems with SendMsg
  $$RefTags {'ERROR'} = $err;
  Gen ($RefTags, 'error.html', 1);
  }
  } #end security
   else {
    #not secure
    $$RefTags {'ERROR'} = 'Some fields have bad value';
   Gen ($RefTags, 'error.html', 1);
   }
}
 }
  elsif ($Finally==2) {
  #drop edi object
  unlink ($self -> {'OBJ_FILENAME'}); #delete tempolary object file
  Gen ($RefTags, 'abort.html', 1);

  } elsif ($self->CompareEdit ($Primary)) {
  #need recreate
  Gen ($RefTags, 'ask.html', 1);
  } else {
  #end edit

  unlink ($self -> {'OBJ_FILENAME'}); #delete tempolary object file

 #test if DEMO version
 if (IF_DEMO ()) {
  Gen ($RefTags, 'demo.html', 1);
 } else {
  $self -> name ($Primary); #change name to primary
  if ($self->Check_Security ()) {
  $self -> Save; #save with new values
  Gen ($RefTags, 'thanks.html', 1);
  } #end security
 else {
    #not secure
    $$RefTags {'ERROR'} = 'Some fields have bad value';
   Gen ($RefTags, 'error.html', 1);
   }

 }
  }
}

#compare objects, if they need recrate rrd file
sub CompareEdit {
 my $self = shift;
 my $Primary = shift;
 my $Result = 0;

 my $obj = Lstatobj::LoadObjectByName ($Primary); # load object to memory
 if (! defined ($obj)) { croak "Can't load object: $Primary \n" }

 #check update period
 if ($self->{'DESC'}{'UPDATE_PERIOD'} ne $obj->{'DESC'}{'UPDATE_PERIOD'} ) {
  $Result = 1;
 } else { $Result = $self->CompareEditObj ($obj) }

 return ($Result); # 1 - need recreate
}

sub CompareEditObj {
 return (0);
}

#show additional info
sub Info {
}
#----------------------------------------------------------------

package TCPUGrapher;
use vars qw ( @ISA );
use Carp;
use Misc;


@ISA = qw ( TGrapher );
sub new {
 my $proto = shift;
 my $class = ref ($proto) || $proto;
 my $self = $class -> SUPER::new ();
 #setting default values
 LoadLangData ('','cpu');
 $self -> name ( 'cpuXX');
 $self -> {'HAS_INFO'} = 1; #this object has additional info
 $self -> {'TITLE_TAG'} = 'CPU_DEFTITLE'; #tag od default title
 $self -> {'HELP_FILE'} = 'cpu.html'; #set help file

 $self -> {'DESC'}{'UPDATE_PERIOD'} = '1m';
 bless ($self, $class);
 return $self;
}

sub Info {
 my $self=shift;
  #show cpuinfo
show_file ($self -> {'CONFIG'}{'PROC_CPUINFO'});
}
sub Create_Object {
 my $self = shift;
 my $rrdstep = shift;
 my $Refsource = shift;

 #for CPU additional  MAX value
 push (@$Refsource, "RRA:AVERAGE:0.5:1:600");
 push (@$Refsource, "RRA:AVERAGE:0.5:6:700");
 push (@$Refsource, "RRA:AVERAGE:0.5:24:775");
 push (@$Refsource, "RRA:AVERAGE:0.5:288:797");
 push (@$Refsource,"RRA:MAX:0.5:1:600");
 push (@$Refsource,"RRA:MAX:0.5:6:700");
 push (@$Refsource,"RRA:MAX:0.5:24:775");
 push (@$Refsource,"RRA:MAX:0.5:288:797");

 push (@$Refsource,"DS:user:COUNTER:$rrdstep:0:U");
 push (@$Refsource,"DS:system:COUNTER:$rrdstep:0:U");
 push (@$Refsource,"DS:idle:COUNTER:$rrdstep:0:U");
  return 1;

}

#update values in RRD file
sub Update_Object {
 my $self = shift;
 my $filename = $self -> {'CONFIG'}{'PROC_STAT_FILE'};
 my $start=time;
 my @value =();
 open (FILE, "<$filename" ) || carp "Can't open file $filename: $! \n";
 my @allvalues=<FILE>;
foreach (@allvalues) {
 if (/^cpu\s/) {
  @value = split;
  last;
  }
}
close FILE;
 $value[1] += $value [2]; #all user usage together
#finally update RRD file
 return ("$start:$value[1]:$value[3]:$value[4]");
}


sub Draw_Object {
 my $self = shift;
 my $rrd =shift;
 my $Refsource =shift;

  push (@$Refsource,"--upper");
  push (@$Refsource,"100");
  push (@$Refsource,"--vertical-label");
  push (@$Refsource,$dict{'CPU_VLABEL'});
  push (@$Refsource,"COMMENT: $dict{CPU_TABLE_CAPTION}");
  push (@$Refsource,"COMMENT:\\s");
  push (@$Refsource,"COMMENT:\\s");

  push (@$Refsource,"DEF:user=$rrd:user:AVERAGE");
  push (@$Refsource,"DEF:system=$rrd:system:AVERAGE");
  push (@$Refsource,"DEF:idle=$rrd:idle:AVERAGE");
  push (@$Refsource,"CDEF:all=user,system,+,idle,+");
  push (@$Refsource,"CDEF:us=user,100,*,all,/");
  push (@$Refsource,"CDEF:sys=system,100,*,all,/");
  push (@$Refsource,"CDEF:alls=us,sys,+");
  push (@$Refsource,"AREA:sys#FF0000:$dict{CPU_SYSTEM}");
  #correct for translations
  my $spacing = " " x (12 -length ($dict{'CPU_SYSTEM'}));
  push (@$Refsource,"COMMENT: $spacing");

  push (@$Refsource,"GPRINT:sys:MAX: \%2.lf");
  push (@$Refsource,"COMMENT:   ");
  push (@$Refsource,"GPRINT:sys:LAST:\%2.lf");
  push (@$Refsource,"COMMENT:\\s");
  push (@$Refsource,"COMMENT:\\s");
  push (@$Refsource,"STACK:us#0000FF:$dict{CPU_USER}");
 $spacing = " " x (12 -length ($dict{'CPU_USER'}));
  push (@$Refsource,"COMMENT: $spacing");

  push (@$Refsource,"GPRINT:us:MAX: \%2.lf");
  push (@$Refsource,"COMMENT:   ");
  push (@$Refsource,"GPRINT:us:LAST:\%2.lf");
  push (@$Refsource,"COMMENT:\\s");
 #push (@sources,"HRULE:50#FF0000:ala");
  push (@$Refsource,"COMMENT:\\s");
  push (@$Refsource,"COMMENT:$dict{CPU_TOTAL} ");
  $spacing = " " x (13 -length ($dict{'CPU_TOTAL'}));
  push (@$Refsource,"COMMENT: $spacing");

  push (@$Refsource,"GPRINT:alls:MAX: \%2.lf");
  push (@$Refsource,"COMMENT:   ");
  push (@$Refsource,"GPRINT:alls:LAST:\%2.lf");
}



#---------------------------------------------------------------
package TMEMGrapher;
use vars qw ( @ISA );
use Carp;
use Misc;

@ISA = qw ( TGrapher );
sub new {
 my $proto = shift;
 my $class = ref ($proto) || $proto;
 my $self = $class -> SUPER::new ();
 LoadLangData ('','mem');
 #setting default values
 $self -> name ( 'memXX');
 $self -> {'TITLE_TAG'} = 'MEM_DEFTITLE';
 $self -> {'HELP_FILE'} = 'mem.html'; #set help file
 bless ($self, $class);
 return $self;
}


sub Create_Object {
 my $self = shift;
 my $rrdstep = shift;
 my $Refsource = shift;

 push (@$Refsource,"DS:total:GAUGE:$rrdstep:0:U");
 push (@$Refsource,"DS:used:GAUGE:$rrdstep:0:U");
 push (@$Refsource,"DS:buffers:GAUGE:$rrdstep:0:U");
 push (@$Refsource,"DS:cached:GAUGE:$rrdstep:0:U");
 push (@$Refsource,"DS:swap:GAUGE:$rrdstep:0:U");
 push (@$Refsource,"DS:swapused:GAUGE:$rrdstep:0:U");
  return 1;

}

#update values in RRD file
sub Update_Object {
 my $self = shift;
 my $filename = $self -> {'CONFIG'}{'PROC_MEM_FILE'};
 my $start=time;
 my @value =();
 my @value2 =();
 open (FILE, "<$filename" ) || carp "Can't open file $filename: $! \n";
 my @allvalues=<FILE>;
 foreach (@allvalues) {
 if (/^Mem:\s/) {
   @value = split;
     }
 elsif (/^Swap:\s/) {
   @value2 = split;
   last;
   }
 }

close FILE;
#finally update RRD file
 return ("$start:$value[1]:$value[2]:$value[5]:$value[6]:$value2[1]:$value2[2]");
}


sub Draw_Object {
 my $self = shift;
 my $rrd =shift;
 my $Refsource =shift;

  push (@$Refsource,"--upper");
  push (@$Refsource,"100");
  push (@$Refsource,"--vertical-label");
  push (@$Refsource,$dict {'MEM_VLABEL'});

  push (@$Refsource,"DEF:total=$rrd:total:AVERAGE");
  push (@$Refsource,"DEF:used=$rrd:used:AVERAGE");
  push (@$Refsource,"DEF:buffers=$rrd:buffers:AVERAGE");
  push (@$Refsource,"DEF:cached=$rrd:cached:AVERAGE");
  push (@$Refsource,"DEF:swap=$rrd:swap:AVERAGE");
  push (@$Refsource,"DEF:swapused=$rrd:swapused:AVERAGE");

  push (@$Refsource,"CDEF:all=total,swap,+");
  push (@$Refsource,"CDEF:prog=used,buffers,-,cached,-");
  push (@$Refsource,"CDEF:progd=prog,100,*,all,/");
  push (@$Refsource,"CDEF:buffersd=buffers,100,*,all,/");
  push (@$Refsource,"CDEF:cachedd=cached,100,*,all,/");
  push (@$Refsource,"CDEF:swapd=swapused,100,*,all,/");

  push (@$Refsource,"CDEF:totalk=total,1024,/");
  push (@$Refsource,"CDEF:usedk=used,1024,/");
  push (@$Refsource,"CDEF:free=totalk,usedk,-");

  push (@$Refsource,"AREA:cachedd#FFFF00:$dict{MEM_CACHE}");
  push (@$Refsource,"STACK:buffersd#00FF00:$dict{MEM_BUFFERS}");
  push (@$Refsource,"STACK:progd#0000FF:$dict{MEM_PROGS}");
  push (@$Refsource,"STACK:swapd#FF0000:$dict{MEM_SWAP}");
  push (@$Refsource,"COMMENT:\\s");
  push (@$Refsource,"COMMENT:\\s");
  push (@$Refsource,"COMMENT:$dict{MEM_TABLE_CAPTION}");
  push (@$Refsource,"COMMENT:\\s");
  push (@$Refsource,"COMMENT:\\s");
  push (@$Refsource,"GPRINT:totalk:LAST:\%8.lf");
  push (@$Refsource,"GPRINT:usedk:LAST:\%6.lf");
  push (@$Refsource,"GPRINT:free:LAST:\%6.lf");

  push (@$Refsource,"CDEF:totals=swap,1024,/");
  push (@$Refsource,"CDEF:useds=swapused,1024,/");
  push (@$Refsource,"CDEF:frees=totals,useds,-");

  push (@$Refsource,"GPRINT:totals:LAST:\%10.lf");
  push (@$Refsource,"GPRINT:useds:LAST:\%6.lf");
  push (@$Refsource,"GPRINT:frees:LAST:\%6.lf");
  push (@$Refsource,"COMMENT:[KB]");
}


#---------END_TMEM------------------------------------------
package TLOADGrapher;
use vars qw ( @ISA );
use Carp;
use Misc;

@ISA = qw ( TGrapher );
sub new {
 my $proto = shift;
 my $class = ref ($proto) || $proto;
 my $self = $class -> SUPER::new ();
 LoadLangData ('','load');
 #setting default values
 $self -> name ( 'loadXX');
 $self -> {'DESC'}{'UPDATE_PERIOD'} = '1m';
 $self -> {'TITLE_TAG'} = 'LOAD_DEFTITLE';
 $self -> {'DESC'}{'LOW_LEVEL'} = 1;
 $self -> {'DESC'}{'HIGH_LEVEL'} = 2;
 $self -> {'DESC'}{'TYPE'} = 0;
 $self -> AddToCHKBOX ('TYPE');
 $self -> {'HAS_INFO'} = 1; #this object has additional info
 $self -> {'HELP_FILE'} = 'load.html'; #set help file
 bless ($self, $class);
 return $self;
}

sub Info { #additional info
 my $self = shift;
 my $uptime = $self -> {'CONFIG'}{'UPTIME'};
 show_command ($uptime);
 $uptime = $self -> {'CONFIG'}{'PROC_VERSION'};
 show_file ($uptime);
 $uptime = $self -> {'CONFIG'}{'REDHAT'};
 show_file ($uptime);
 }

sub Create_Object {
 my $self = shift;
 my $rrdstep = shift;
 my $Refsource = shift;

 push (@$Refsource,"DS:load:GAUGE:$rrdstep:0:U");
  return 1;

}

#update values in RRD file
sub Update_Object {
 my $self = shift;
 my $filename = $self -> {'CONFIG'}{'PROC_LOAD_FILE'};
 my $start=time;
 my @value =();
 open (FILE, "<$filename" ) || carp "Can't open file $filename: $! \n";
 my $allvalues=<FILE>;
  @value = split ('\s',$allvalues);

 close FILE;
#finally update RRD file
 return ("$start:$value[0]");
}


sub Draw_Object {
 my $self = shift;
 my $rrd =shift;
 my $Refsource =shift;

 my  $typ= $self-> {'DESC'}{'TYPE'} ; #type of graph
 my $lowlevel=$self-> {'DESC'}{'LOW_LEVEL'} ; #low level od system load
 my $highlevel=$self-> {'DESC'}{'HIGH_LEVEL'} ; #jigh level od system load

  push (@$Refsource,"--vertical-label");
  push (@$Refsource,$dict{LOAD_VLABEL});

  push (@$Refsource,"DEF:load=$rrd:load:AVERAGE");

 if ($typ) {
 push (@$Refsource,"CDEF:loadok=load,$lowlevel,MIN");
 push (@$Refsource,"CDEF:loadwarn=load,$lowlevel,LT,UNKN,load,$lowlevel,-,$highlevel,$lowlevel,-,MIN,IF");
 push (@$Refsource,"CDEF:loadhot=load,$highlevel,LT,UNKN,load,$highlevel,-,IF");
 } else {
 push (@$Refsource,"CDEF:loadok=load,POP,$lowlevel,0,-");
 push (@$Refsource,"CDEF:loadwarn=load,POP,$highlevel,$lowlevel,-");
 push (@$Refsource,"CDEF:loadhot=load,POP,INF");
 }
 push (@$Refsource,"AREA:loadok#00FF00:$dict{LOAD_LOW}");
 push (@$Refsource,"STACK:loadwarn#FFFF00:$dict{LOAD_MED}");
 push (@$Refsource,"STACK:loadhot#FF0000:$dict{LOAD_HI}");
 push (@$Refsource,"LINE2:load#000000:$dict{LOAD_VAL}") if (! $typ);
 push (@$Refsource,"GPRINT:load:LAST:$dict{LOAD_LAST} \%2.2lf");
}


#edit additional primary fields
sub EditObjPrimary {
my $self = shift;
my $RefTags = shift;

#show additional template for this object
Misc::Gen ($RefTags, 'eloadpri.html',1);
}


#---------End-TLOAD-----------------------------------------
package TDISKGrapher;
use vars qw ( @ISA );
use Carp;
use Misc;

@ISA = qw ( TGrapher );
sub new {
 my $proto = shift;
 my $class = ref ($proto) || $proto;
 my $self = $class -> SUPER::new ();
 LoadLangData ('','disk');
 #setting default values
 $self -> name ( 'diskXX');
 $self -> {'TITLE_TAG'} = 'DISK_DEFTITLE';
 $self -> {'DESC'}{'TYPE'} = 1;
 $self -> AddToRADIOBOX ('TYPE'); #for radio box
 $self -> {'DESC'}{'DISK'} = "";
 $self -> {'HELP_FILE'} = 'disk.html'; #set help file
 bless ($self, $class);
 return $self;
}

sub Create_Object {
 my $self = shift;
 my $rrdstep = shift;
 my $Refsource = shift;

 push (@$Refsource,"DS:read:COUNTER:$rrdstep:0:1E6");
 push (@$Refsource,"DS:write:COUNTER:$rrdstep:0:1E6");
  return 1;

}


#for disk stats
my %idemajor=(3,'a',
              22,'c',
              33,'e',
              34,'g',
              56,'i',
              57,'k');

my $scsimajor=8;
my %disktable = (); #list with disks data
my %validdisk = (); #table valid disks
my $diskcollected=0;
my $diskcollectstart;
my $lastrunme=0;

sub Create_valid_disks {
#build list valid disks
my $procpart = $config{'PROC_PART_FILE'};
open (FILE, "<$procpart") || die "Cant open $procpart: $! \n";
my @stat = <FILE>;
close FILE;

foreach my $line ( @stat) {
 $line =~ s/^\s+//g;
 if ($line =~ /^\d+/ ) {
  my ($major, $minor, $blocks, $name, $rest) = split (/\s+/,$line,5);
  if ($name =~ /^[sh]d[a-z]+$/) {
  my $disknum = ord (substr ($name, 2, 1)) - ord 'a';
  $validdisk {$name} = [ $major, $minor, $disknum, $blocks];
  }
 }
}

}




#collect diskstats
sub diskcollect {
if ($lastrunme != $lastrun) {
$lastrunme = $lastrun;
$diskcollectstart = time;
 if (! $diskcollected ) { #create table valid disks
  Create_valid_disks ();
  $diskcollected=1;
 }

#collect disk data
my $procstat = $config{'PROC_STAT_FILE'};
open (FILE, "<$procstat") || die "Cant open $procstat: $! \n";
my @stat = <FILE>;
close FILE;
#for kernel 2.2. style
my @olddiskstat;

foreach (@stat) {
 if (/^disk_io:/){
  while (/\((\d+)\,(\d+)\)\:\((\d+)\,(\d+)\,(\d+)\,(\d+)\,(\d+)\)/g){
   #check for ide
   if (defined ($idemajor{$1})) {
    #disk ide
    my $disknr = $2;
    if ($disknr > 1) {
     #try workaround kernel 2.4 bug
     $disknr = $disknr - 2;
    }
    my $letter = chr ( ord ($idemajor{$1}) + $disknr);
    $disktable {"hd$letter"} = [$3,$4,$5,$6,$7];

   } elsif ($1 == $scsimajor) {
    my $letter = chr ( ord ('a') + $2);
    $disktable {"sd$letter"} = [$3,$4,$5,$6,$7];
    }
  }
  last;
 } elsif (/^disk[\s\_]/ ) {
   #kernel 2.2.X style
   s/^\w+\s//g; #delete first chars
   my @diskline = split;
   push (@olddiskstat, [@diskline]) ;
  }

}

if (@olddiskstat) {
 #for kernels 2.2.X
 foreach my $diskname (keys %validdisk ) {
  my $ref = $validdisk {$diskname};
  my $disknum = $$ref [2];
  if (defined ($olddiskstat[4] [ $disknum ])) {
   $disktable {$diskname} = [$olddiskstat[0][$disknum],
                            $olddiskstat[1][$disknum],
			    $olddiskstat[3][$disknum],
			    $olddiskstat[2][$disknum],
			    $olddiskstat[4][$disknum]];
  } else { &errorproc ("Not valid diskname: $diskname\n");
   }
 }

}

}
}



#update values in RRD file
sub Update_Object {
 my $self = shift;

 &diskcollect; #collect disk data
 my $diskname = $self -> {'DESC'}{'DISK'};
 my $refdata = ['U','U','U','U','U'];
 if (defined ($disktable{$diskname})) {
  #found disk
  $refdata = $disktable{$diskname};
 }


my $data;

if ($self -> {'DESC'}{'TYPE'}) {
$data ="$diskcollectstart:$$refdata[2]:$$refdata[4]";
} else {
$data ="$diskcollectstart:$$refdata[1]:$$refdata[3]";
}

#finally update RRD file
 return ($data);
}


sub Draw_Object {
 my $self = shift;
 my $rrd =shift;
 my $Refsource =shift;

 my  $typ= $self-> {'DESC'}{'TYPE'} ; #type of graph
 my $diskname=$self-> {'DESC'}{'DISK'} ; #low level od system load

 push (@$Refsource,"--vertical-label");

if ($typ) {
  push (@$Refsource,$dict{DISK_VLABEL_BLOCK});
 } else {
  push (@$Refsource,$dict{DISK_VLABEL_OPER});
 }

  push (@$Refsource,"DEF:read=$rrd:read:AVERAGE");
  push (@$Refsource,"DEF:write=$rrd:write:AVERAGE");
  push (@$Refsource,"AREA:read#0000FF:$dict{DISK_READ}");
  push (@$Refsource,"STACK:write#FF0000:$dict{DISK_WRITE}");

}


#edit additional primary fields
sub EditObjPrimary {
my $self = shift;
my $RefTags = shift;

#show additional template for this object
Misc::Gen ($RefTags, 'ediskpri.html',1);
}

#check if need recreate RRD file
sub CompareEditObj {
 my $self = shift;
 my $newobj = shift;
 my $Result = 0;
 $Result = 1 if ($self-> {'DESC'}{'TYPE'} ne $newobj-> {'DESC'}{'TYPE'});
 $Result = 1 if ($self-> {'DESC'}{'DISK'} ne $newobj-> {'DESC'}{'DISK'});
 return ($Result);
}


#---------End-TDISK-----------------------------------------

package TINTGrapher;
use vars qw ( @ISA );
use Carp;
use Misc;
use Misc qw ( show_command ConvB );

my %collect; #table clolected data
my $lastrunme1=-10;


@ISA = qw ( TGrapher );
sub new {
 my $proto = shift;
 my $configref = shift;
 my $class = ref ($proto) || $proto;
 my $self = $class -> SUPER::new ($configref);
 LoadLangData ('','int');
 #setting default values
 $self -> name ( 'intXX');
 $self ->{'COLLECT'} = \%collect;
 $self -> {'TITLE_TAG'} = 'INT_DEFTITLE';
 $self -> {'DESC'}{'INTERFACE'} = shift;
 $self -> {'DESC'}{'LIMIT'} = 0;
 $self -> {'DESC'}{'SPEED'} = 1; #0 < 1Mbit/s , 1 - 10 , 2 -100, 3 - undef
 $self -> {'DESC'}{'TYPE'} = 0;     #0 bytes or 1 packets
 $self -> {'DESC'}{'BITS'} = 1;     #0 bytes/s or 1 bits/s
 $self->AddToRADIOBOX ('SPEED','TYPE','BITS'); #for editing as radio box
 $self -> {'HELP_FILE'} = 'int.html'; #set help file

 $self -> {'HAS_INFO'} = 1; #this object has additional info

# $self -> {'UPDATE_PERIOD'} = '5m';
 bless ($self, $class);
 return $self;
}

sub Info { #additional info
 my $self = shift;
 my $ifconfig = $self -> {'CONFIG'}{'IFCONFIG'};
 show_command ($ifconfig);
 }

sub Create_Object {
 my $self = shift;
 my $rrdstep = shift;
 my $Refsource = shift;

 my $speed = $self -> {'DESC'}{'SPEED'};
 if ($speed == 3) { $speed='U'}
  else {$speed = int (1E5 * (10 ** $speed)) };
 if (($self -> {'DESC'}{'TYPE'}) && ($speed ne 'U') ) { $speed = $speed * 0.015 }

 push (@$Refsource,"DS:input:COUNTER:$rrdstep:0:$speed");
 push (@$Refsource,"DS:output:COUNTER:$rrdstep:0:$speed");
  return 1;

}


#collecting informations about interfaces
sub Get_Interface_Data {
 if ($lastrunme1 != $lastrun) {
 $collect{start} = time; #save time;
 #open file with data
 my $filename=$config {'PROC_INT_FILE'};
 open (FILE, "<$filename" ) || carp "Can't open file $filename: $! \n";
 while (<FILE>) {
  if (/:/) {
  my ($name, $data) = split (/:/,$_);
  $name =~ s/^\s+//g;
  $data =~ s/^\s+//g;
  my @data2 = split (/\s+/,$data);
  my @int_data = ();
  push (@int_data,$data2[0]); #bytes received
  push (@int_data,$data2[8]); #bytes sent
  push (@int_data,$data2[1]); #pakcket received
  push (@int_data,$data2[9]); #packet sent
  $collect{$name} = [@int_data]; #add to interface data;
  }
 }
 close FILE;
 $lastrunme1 = $lastrun;
 }
}


#update values in RRD file
sub Update_Object {
 my $self = shift;
 #collect data about interfaces
 Get_Interface_Data ();

 my  $typ = $self-> {'DESC'}{'TYPE'} ; #type of graph packets or bytes
 my  $interface = $self-> {'DESC'}{'INTERFACE'} ; #interface name
 my  $collect = $self-> {'COLLECT'} ; #reference to collected data
 my @daneint = ("U","U","U","U");
 my $refdane = \@daneint;
 if (defined $$collect{$interface}) {
  #interface found
  $refdane = $$collect{$interface};
 };

 my $data;

 if ($typ) {  #packets
  $data ="$$collect{start}:$$refdane[2]:$$refdane[3]";
} else {     #bytes
  $data = "$$collect{start}:$$refdane[0]:$$refdane[1]";
}

#finally update RRD file
 return ($data);
}


sub Draw_Object {
 my $self = shift;
 my $rrd =shift;
 my $Refsource =shift;

 my  $typ = $self-> {'DESC'}{'TYPE'} ; #type of graph packets or bytes
 my  $limit = $self-> {'DESC'}{'LIMIT'} ; #limit of values
 my  $bits = $self-> {'DESC'}{'BITS'} ; #limit of values
 my $mult=8; #for multiple bytes

 push (@$Refsource,"--vertical-label");
 #eval some statistic
 my $refdata = $self->ExtraData ();


 if ($typ) {
  push (@$Refsource,$dict {INT_VLABEL_PAC});
 } else {
  if ($bits) {push (@$Refsource,$dict {INT_VLABEL_BIT})}
   else {push (@$Refsource,$dict{INT_VLABEL_BYTE}); $mult=1;}
 }

  push (@$Refsource,"DEF:input=$rrd:input:AVERAGE");
  push (@$Refsource,"DEF:output=$rrd:output:AVERAGE");
  push (@$Refsource,"CDEF:total=input,output,+");
if ($limit) {
if ($typ) {
push (@$Refsource,"CDEF:in=input,DUP,$limit,LT,EXC,$limit,IF");
push (@$Refsource,"CDEF:out=output,DUP,$limit,LT,EXC,$limit,IF");
push (@$Refsource,"CDEF:all=total,DUP,$limit,LT,EXC,$limit,IF");

} else {
push (@$Refsource,"CDEF:in=input,$mult,*,DUP,$limit,LT,EXC,$limit,IF");
push (@$Refsource,"CDEF:out=output,$mult,*,DUP,$limit,LT,EXC,$limit,IF");
push (@$Refsource,"CDEF:all=total,$mult,*,DUP,$limit,LT,EXC,$limit,IF");
}
}
 else {
#without limit
if ($typ){
push (@$Refsource,"CDEF:in=input");
push (@$Refsource,"CDEF:out=output");
push (@$Refsource,"CDEF:all=total");
} else {
push (@$Refsource,"CDEF:in=input,$mult,*");
push (@$Refsource,"CDEF:out=output,$mult,*");
push (@$Refsource,"CDEF:all=total,$mult,*");
}
}

  my $out_data = $refdata -> {'output'}; #stats for output
  my $in_data = $refdata -> {'input'}; #stats for output
  my @total_data;
  my $labelsize=8;
  #add input and output statistic
  for (my $i=0; $i<@$out_data;$i++) { push (@total_data, $in_data->[$i] + $out_data->[$i])}

  push (@$Refsource,"COMMENT:$dict{'INT_TABLE'}\\s");
  push (@$Refsource,"COMMENT:\\s");
if ($typ) {
  #packet
  push (@$Refsource,"COMMENT:                 [pkt]  [p/s]  [p/s]  [p/s]  [p/s]\\s");
} else
 { #bytes
   push (@$Refsource,"COMMENT:                  [B]  [B/s]  [B/s]  [B/s]  [B/s]\\s");
 }
  push (@$Refsource,"COMMENT:\\s");
  #correct for translations
  my $spacing = " " x ($labelsize -length ($dict{'INT_TOTAL'}));
  push (@$Refsource,"AREA:all#0000FF:$dict{INT_TOTAL}");
  push (@$Refsource,"COMMENT: $spacing");

  my $data = \@total_data;
  for (my $i=0; $i<@$data;$i++) { $data->[$i] = ConvB ($data->[$i])}

  push (@$Refsource,"COMMENT:$data->[0] $data->[1] $data->[3] $data->[2] $data->[4]");

  push (@$Refsource,"COMMENT:\\s");
  push (@$Refsource,"COMMENT:\\s");

 push (@$Refsource,"LINE2:out#00FF00:$dict{INT_OUTPUT}");
  $spacing = " " x ($labelsize -length ($dict{'INT_OUTPUT'}));
  push (@$Refsource,"COMMENT: $spacing");

  $data = $out_data; #stats for output
  #convert to value with prefix
  for (my $i=0; $i<@$data;$i++) { $data->[$i] = ConvB ($data->[$i])}

  push (@$Refsource,"COMMENT:$data->[0] $data->[1] $data->[3] $data->[2] $data->[4]");

  push (@$Refsource,"COMMENT:\\s");
  push (@$Refsource,"COMMENT:\\s");


push (@$Refsource,"LINE2:in#FF0000:$dict{INT_INPUT}");
  $spacing = " " x ($labelsize -length ($dict{'INT_INPUT'}));
  push (@$Refsource,"COMMENT: $spacing");
  $data = $in_data; #stats for input
  #convert to value with prefix
  for (my $i=0; $i<@$data;$i++) { $data->[$i] = ConvB ($data->[$i])}

  push (@$Refsource,"COMMENT:$data->[0] $data->[1] $data->[3] $data->[2] $data->[4]");

}



#edit additional primary fields
sub EditObjPrimary {
my $self = shift;
my $RefTags = shift;

#show additional template for this object
Misc::Gen ($RefTags, 'eintpri.html',1);
}

#check if need recreate RRD file
sub CompareEditObj {
 my $self = shift;
 my $newobj = shift;
 my $Result = 0;
 $Result = 1 if ($self-> {'DESC'}{'TYPE'} ne $newobj-> {'DESC'}{'TYPE'});
 $Result = 1 if ($self-> {'DESC'}{'SPEED'} ne $newobj-> {'DESC'}{'SPEED'});
 return ($Result);
}



#---------END TINTGRAPHER-----------------------------------
package TPARTGrapher;
use vars qw ( @ISA );
use Misc;
use Misc qw ( @defcolour );
@ISA = qw ( TGrapher );
sub new {
 my $proto = shift;
 my $class = ref ($proto) || $proto;
 my $self = $class -> SUPER::new ();
 LoadLangData ('','part');
 #setting default values
 $self -> name ( 'partXX');
 $self -> {'TITLE_TAG'} = 'PART_DEFTITLE';
 my @PART;
 $self -> {'DESC'}{'ADV'} = [ @PART ]; #reference to partition info
 $self -> {'HAS_INFO'} = 1; #this object has additional info
 $self -> {'HELP_FILE'} = 'part.html'; #set help file
 bless ($self, $class);
 return $self;
}

sub Info { #additional info
 my $self = shift;
 my $partfile = $self -> {'CONFIG'}{'PROC_PART_FILE'};
 if (open (FILE, "<$partfile") && (my @content2=<FILE>) && (close FILE) ) {
  print "<TABLE BORDER=1 BORDERCOLOR=#000000 CELLSPACING=0>\n";
  my $first=1;
  foreach my $line (@content2) {
   chomp ($line);
   $line =~ s/^\s+//g;
   if ($line) {
    if ($first) {
    print "<TR BGCOLOR=#A0A0FF ALIGN=RIGHT>\n";
    $first = 0;
    } else
     {print "<TR ALIGN=RIGHT>\n";}
    my @cont = split (/\s+/,$line);
    foreach my $value (@cont) {
     print "<TD>$value</TD>";
    }
    print "</TR>\n";
   }
  }
   print "</TABLE>\n";
 }
 }


sub Create_Object {
 my $self = shift;
 my $rrdstep = shift;
 my $Refsource = shift;

 my $PartRef =  $self -> {'DESC'}{'ADV'}; #reference to array of partitions
 if (@$PartRef) { #has any elements
 for (my $i=0; $i < @$PartRef; $i++) {
  push (@$Refsource,"DS:total$i:GAUGE:$rrdstep:0:U"); #for each mount point
  push (@$Refsource,"DS:usage$i:GAUGE:$rrdstep:0:U");
  push (@$Refsource,"DS:free$i:GAUGE:$rrdstep:0:U");
  }
 return 1;
 } else {return 0}

}

#prepare to edit partitions info
sub EditObjAdvanced {
 my $self = shift;
 my $RefTags = shift;

 my $PartRef =  $self -> {'DESC'}{'ADV'}; #reference to array of partitions

 Gen ($RefTags, 'ebegadv.html', 1);
 if (@$PartRef) { #has any element;
  print "<TABLE BORDER\=1 BGCOLOR=\"#CACAFF\" CELLSPACING=0 CELLPADDING=0 >\n";
  print "<TR BGCOLOR=\"#A0A0FF\"><B><TD><FONT COLOR=red>*</FONT>$dict{PART_EDIT_POS}</TD><TD>\n";
  print "$dict{PART_EDIT_MOUNT}</TD><TD>$dict{PART_EDIT_COLOR}</TD><TD>$dict{PART_EDIT_DESC}</TD></B></TR>\n";
  for (my $i=0; $i<@$PartRef; $i++) {
   my $HashRef = $$PartRef [$i];
   print "<TR><TD><INPUT TYPE=CHECKBOX NAME=\"$i\" ></TD>\n";
   print "<TD><INPUT TYPE=\"text\" NAME=\"MOUNT$i\" VALUE=\"";
   print $$HashRef {'MOUNT'};
   print "\" SIZE=20 MAXLENGTH=40></TD>\n";

   print "<TD><INPUT TYPE=\"text\" NAME=\"COLOR$i\" VALUE=\"";
   print $$HashRef {'COLOR'};
   print "\" SIZE=8 MAXLENGTH=8></TD>\n";

   print "<TD><INPUT TYPE=\"text\" NAME=\"DESC$i\" VALUE=\"";
   print $$HashRef {'DESC'};
   print "\" SIZE=20 MAXLENGTH=60></TD>\n";

   print "</TR>\n";
  }
  print "</TABLE>\n";
 }
 Gen ($RefTags, 'eendadv.html', 1);
}

sub PostEditObjAdvanced {
 my $self = shift;
 my $RefPostTags =shift;
 my $Result=0;

 #catch end form's action
 $Result = $RefPostTags->{'RESULT'} if ( defined ($RefPostTags->{'RESULT'}));

 if (( $Result eq 'NEW_EL') or #run only if advanced data was edited
     ( $Result eq 'DEL_EL') or
     ( $Result eq 'OK_EL') ) {
 my @NewPart;
 $self -> {'DESC'}{'ADV'} = [ @NewPart ]; #reference to array of partitions
 my $PartRef = $self -> {'DESC'}{'ADV'};
 #save all elements to array
 my $i = 0;
 my $del = ( $Result eq 'DEL_EL') ; #if delete any element
 while (defined ($RefPostTags->{"MOUNT$i"}) ) {
  if (! (defined ($RefPostTags->{$i}) && $del )) {
  my %row;   #add to data
  $row {'MOUNT'} = $RefPostTags->{"MOUNT$i"};
  $row {'COLOR'} = $RefPostTags->{"COLOR$i"};
  $row {'DESC'} = $RefPostTags->{"DESC$i"};
  push (@$PartRef, { %row } );
  }
  $i++;
 }

 if ( $Result eq 'NEW_EL') {
  #pressed New Element
  my %NewHash;
  $NewHash {'MOUNT'} = "";
  $NewHash {'COLOR'} = "";
  $NewHash {'DESC'} = "";
  push (@$PartRef, { %NewHash });
 }
}
}

#check if need recreate RRD file
sub CompareEditObj {
 my $self = shift;
 my $newobj = shift;
 my $Result = 0;
 my $newobjpart = $newobj-> {'DESC'}{'ADV'};
 my $objpart = $self-> {'DESC'}{'ADV'};
 if ($#$newobjpart != $#$objpart ) {
  $Result=1; #difference in elements count
   }
 return ($Result);
}


#update values in RRD file
sub Update_Object {
 my $self = shift;
 my $data = "";
 my  %Info;  #array to collect info

my $df = $self -> {'CONFIG'}{'DF'}; #path to df command
if (-x $df ) {
#start collect info about mounted partitions
my $start = time;
my @mounted =`$df`;
#pozbycie sie pierwszej linijki opisu
shift @mounted; #without first line
foreach (@mounted) {
 my @point = split;
 my %newpoint;
 my $pointname = $point [5];
 $newpoint {'total'}= $point [1];
 $newpoint{'used'} = $point [2];
 $newpoint{'free'} = $point [3];
 $Info {$pointname} = {%newpoint};
 }

$data= $start;
my $refPart = $self->{'DESC'}{'ADV'}; #reference to data for collect

foreach my $row (@$refPart ){
  #default values
  my ($tot,$usa,$fre)=(0,0,0);
  if (defined ($Info {$row->{'MOUNT'}})) {
  #found in collected data
   my $ref = $Info {$row->{'MOUNT'}};
   $tot=$ref ->{'total'};
   $usa=$ref ->{'used'};
   $fre=$ref ->{'free'};
   }
   $data .=":$tot:$usa:$fre";
  }
#finally update RRD file
}
 return ($data);
}


sub Draw_Object {
 my $self = shift;
 my $rrd =shift;
 my $Refsource =shift;


 my $curcolour=0;

  push (@$Refsource,"--upper");
  push (@$Refsource,"100");
  push (@$Refsource,"--vertical-label");
  push (@$Refsource,$dict{PART_VLABEL});
  push (@$Refsource,"COMMENT:$dict{PART_TABLE_CAPTION}");

  my $RefPart=$self->{'DESC'}{'ADV'};
  if (@$RefPart) {
  #foreach mount point
  for (my $i=0; $i<@$RefPart; $i++) {

  my $row = $$RefPart[$i];

  push (@$Refsource,"DEF:total$i=$rrd:total$i:AVERAGE");
  push (@$Refsource,"DEF:usage$i=$rrd:usage$i:AVERAGE");
  push (@$Refsource,"DEF:free$i=$rrd:free$i:AVERAGE");
  push (@$Refsource,"CDEF:part$i=usage$i,100,*,total$i,/");
 # my $kas=/"/";
  push (@$Refsource,"COMMENT:\\s");
  push (@$Refsource,"COMMENT:\\s");

  #select colour
  my $colour;
  if ($row->{'COLOR'}) {
  $colour = $row->{'COLOR'};
 } else {
  $colour = $defcolour[$curcolour++];
  }

  my $mountpoint = $row->{'MOUNT'};
  push (@$Refsource,"LINE2:part$i\#$colour:$mountpoint");
  my $mountdesc = $row->{'DESC'};
  $mountdesc='' if (! defined ($mountdesc));

  my $mountdesc1 = " " x (12 -length ($mountpoint)).$mountdesc;
  $mountdesc1 .= " " x (12 -length ($mountdesc));

  push (@$Refsource,"COMMENT:$mountdesc1");
  push (@$Refsource,"GPRINT:total$i:LAST:\%10.lf");
  push (@$Refsource,"GPRINT:usage$i:LAST:\%10.lf");
  push (@$Refsource,"GPRINT:free$i:LAST:\%10.lf");
  } #end for
 }
}
#---------END TPARTGRAPHER-----------------------------------
package TIPGrapher;
use vars qw ( @ISA );
use Misc;
use Misc qw ( @defcolour ConvB );
@ISA = qw ( TGrapher );
sub new {
 my $proto = shift;
 my $class = ref ($proto) || $proto;
 my $self = $class -> SUPER::new ();
 LoadLangData ('','ip');
 #setting default values
 $self -> name ( 'ipXX');
 $self -> {'TITLE_TAG'} = 'IP_DEFTITLE';
 my @IP;
 $self -> {'DESC'}{'ADV'} = [ @IP ]; #reference to statistic info
 $self -> {'DESC'}{'SPEED'} = 1; #max speed see TINTGraph
 $self -> {'DESC'}{'TYPE'} = 0;     #0 bytes or 1 packets
 $self -> {'DESC'}{'BITS'} = 1;     #0 bytes pers or 1 bits pers
 $self -> {'DESC'}{'BAR'} = 1;     #0 line or 1 bar
 $self -> {'DESC'}{'EXTRA_INFO'} = 1;     # 1 for extra info
 $self->AddToRADIOBOX ('SPEED','TYPE','BITS','BAR'); #for editing as radio box
 $self->AddToCHKBOX ('EXTRA_INFO'); #for editing as check box
 $self -> {'HELP_FILE'} = 'ip.html'; #set help file

 bless ($self, $class);
 return $self;
}

sub Create_Object {
 my $self = shift;
 my $rrdstep = shift;
 my $Refsource = shift;

 my $PartRef =  $self -> {'DESC'}{'ADV'}; #reference to array of partitions

 my $speed = $self -> {'DESC'}{'SPEED'};
 if ($speed == 3) { $speed='U'}
  else {$speed = int (1E5 * (10 ** $speed)) };
 if (($self -> {'DESC'}{'TYPE'}) && ($speed ne 'U') ) { $speed = $speed * 0.015 }

if (@$PartRef) { #has any elements
 for (my $i=0; $i < @$PartRef; $i++) {
   push (@$Refsource,"DS:m$i:COUNTER:$rrdstep:0:$speed");
 }
 return 1;
 } else {return 0}

}

#edit additional primary fields
sub EditObjPrimary {
my $self = shift;
my $RefTags = shift;

#show additional template for this object
Misc::Gen ($RefTags, 'eippri.html',1);
}


#prepare to edit partitions info
sub EditObjAdvanced {
 my $self = shift;
 my $RefTags = shift;

 my $PartRef =  $self -> {'DESC'}{'ADV'}; #reference to array of partitions

 Gen ($RefTags, 'ebegadv.html', 1);
 if (@$PartRef) { #has any element;
  print "<TABLE BORDER\=1 BGCOLOR=\"#CACAFF\" CELLSPACING=0 CELLPADDING=0 >\n";
  print "<TR BGCOLOR=\"#A0A0FF\"><B><TD><FONT COLOR=red>*</FONT>$dict{IP_EDIT_POS}</TD><TD>\n";
  print "$dict{IP_EDIT_FILTR}</TD><TD>$dict{IP_EDIT_COLOR}</TD><TD>$dict{IP_EDIT_DESC}</TD></B></TR>\n";
  for (my $i=0; $i<@$PartRef; $i++) {
   my $HashRef = $$PartRef [$i];
   print "<TR><TD><INPUT TYPE=CHECKBOX NAME=\"$i\" ></TD>\n";
   print "<TD><INPUT TYPE=\"text\" NAME=\"MARK$i\" VALUE=\"";
   print $$HashRef {'MARK'};
   print "\" SIZE=20 MAXLENGTH=40></TD>\n";

   print "<TD><INPUT TYPE=\"text\" NAME=\"COLOR$i\" VALUE=\"";
   print $$HashRef {'COLOR'};
   print "\" SIZE=8 MAXLENGTH=8></TD>\n";

   print "<TD><INPUT TYPE=\"text\" NAME=\"DESC$i\" VALUE=\"";
   print $$HashRef {'DESC'};
   print "\" SIZE=20 MAXLENGTH=60></TD>\n";

   print "</TR>\n";
  }
  print "</TABLE>\n";
 }
 Gen ($RefTags, 'eendadv.html', 1);
}

sub PostEditObjAdvanced {
 my $self = shift;
 my $RefPostTags =shift;

 my $Result=0;
 #catch end form's action
 $Result = $RefPostTags->{'RESULT'} if ( defined ($RefPostTags->{'RESULT'}));

 if (( $Result eq 'NEW_EL') or #run only if advanced data was edited
     ( $Result eq 'DEL_EL') or
     ( $Result eq 'OK_EL') ) {
 my @NewPart;
 $self -> {'DESC'}{'ADV'} = [ @NewPart ]; #reference to array of partitions
 my $PartRef = $self -> {'DESC'}{'ADV'};
 #save all elements to array
 my $i = 0;
 my $del = ($Result eq 'DEL_EL'); #if delete any element
 while (defined ($RefPostTags->{"MARK$i"}) ) {
  if (! (defined ($RefPostTags->{$i}) && $del )) {
  my %row;   #add to data
  $row {'MARK'} = $RefPostTags->{"MARK$i"};
  $row {'COLOR'} = $RefPostTags->{"COLOR$i"};
  $row {'DESC'} = $RefPostTags->{"DESC$i"};
  push (@$PartRef, { %row } );
  }
  $i++;
 }

 if ($Result eq 'NEW_EL') {
  #pressed New Element
  my %NewHash;
  $NewHash {'MARK'} = "";
  $NewHash {'COLOR'} = "";
  $NewHash {'DESC'} = "";
  push (@$PartRef, { %NewHash });
 }
}
}

#check if need recreate RRD file
sub CompareEditObj {
 my $self = shift;
 my $newobj = shift;
 my $Result = 0;
 my $newobjpart = $newobj-> {'DESC'}{'ADV'};
 my $objpart = $self-> {'DESC'}{'ADV'};
 if ($self->{'DESC'}{'SPEED'} != $newobj->{'DESC'}{'SPEED'} ) {$Result=1}
 elsif ($self->{'DESC'}{'TYPE'} != $newobj->{'DESC'}{'TYPE'} ) {$Result=1}
 elsif ($#$newobjpart != $#$objpart ) {
  $Result=1; #difference in elements count
   }
 return ($Result);
}

my $lastrunme2=-2;
my %collect_mark =(); #table of collected marks
my $ipchains ="";
my $iptables="";
$ipchains = $config{'IPCHAINS'} if (defined ($config{'IPCHAINS'})); #path to ipchains command
$iptables = $config{'IPTABLES'} if (defined ($config{'IPTABLES'}));; #path to iptables command


#parse RefInfo and extract filters to RefData
#mangle for iptables
sub collect_data {
my $RefData = shift;
my $RefInfo = shift;
my $mangle = shift;

$mangle = 0 if (! defined ($mangle));

  my ($mark, $bytes, $pkts, $markpos, $bytespos, $pktspos, $curchain,$curline);
  foreach my $line ( @$RefInfo) {
  next if ($line =~ /^\s*$/);
#  print "$line\n";
 if ($line =~ /^Chain\s+(\S+)/) { #extrct new chain
  $curchain = $1;
  $curchain .= '_M' if ($mangle == 1); #for iptables and mangle
  $curchain .= '_N' if ($mangle == 2); #for iptables and nat

  if ($line =~ /\(policy\s+\w+\:?\s*(\d+)\s+\w+\,\s*(\d+)/) {   #main chain, test it
#   print ("$curchain",$curline,"   ", $line);
   $curline=0;
   $RefData ->{$curchain.$curline} = [$1, $2]; #save pkts, $bytes
  }
   $curline=1;
 }elsif ($line =~ /^\s*pkts/) {  #line with description
  $markpos = index ($line, 'mark');
  $bytespos = index ($line, 'bytes');
  $pktspos = index ($line, 'pkts');
}  elsif  ($line =~ /^\s*\d+/) {
 #line contains valid data
  if ( $line =~ /^\s*(\d+)\s+(\d+)/) {;
    $pkts = $1;
    $bytes = $2;
 } else { #for high value ipchans counter
  $pkts = substr ($line,0, $pktspos +4);
  $pkts =~ s/\s+//g;
  $bytes = substr ($line, $pktspos +4, $bytespos-$pktspos+1);
  $bytes =~ s/\s+//g;
  }
  if ($line =~ /MARK set (0x\w+)/) { #iptables version
   $mark = $1;
  } else {
   $mark = substr ($line, $markpos, 10); #for ipchains test if mark exists
  }
  if  ($mark =~/^0x/) {
  #has mark
  $mark =~ s/\s+//g;
  $mark = 'm'.hex ($mark);
  if (defined ($RefData ->{$mark})) {
  #add to exists
  my $ref = $RefData ->{$mark};
  $ref->[0] += $pkts;
  $ref->[1] += $bytes;
  } else {
    $RefData ->{$mark} = [$pkts, $bytes];
     }
  } #end mark ok
 $RefData ->{$curchain.$curline++} = [$pkts, $bytes]; #save pkts, $bytes
 } #valid data
 } #end for
} #end sub


#collect data about marked ip chain
sub collectmarks {

 if ($lastrunme2 != $lastrun) {
 my $forceIPCHAINS= $config {'FORCE_IPCHAINS'}; #if force IPCHAINS using
 my ($markpos, $bytespos, $pktspos);
 $lastrunme2 = $lastrun;
 %collect_mark = (); #forget all elements
  #store collect time
 $collect_mark{'mstart'} = time;
 my @info;

  #test if use iptable
  if ( ( ! $forceIPCHAINS) && (-x $iptables) ) {
  #iptables version
 @info = `$iptables -L -n -v -x`;
 # my $kas=/"`"
  if ($? > 0) {
   print "Problem with iptables ($iptables): $! \n";
   return;
  }
 collect_data (\%collect_mark, \@info);
 @info = `$iptables -L -n -v -t mangle -x`;
 # my $kas=/"`"
 collect_data (\%collect_mark, \@info,1); #for mangle and MARKs


 } #end iptables
  else
  {
   if ( -x $ipchains ) {
  #ipchains version
 @info = `$ipchains -L -n -v -x`;
 # my $kas=/"`"
  if ($? > 0) {
   print "Problem with ipchains ($ipchains): $! \n";
   return;
  }
 collect_data (\%collect_mark, \@info);
  } #end ipchains
 } #end else
}#lastrunme
}#collectmarks



#update values in RRD file
sub Update_Object {
 my $self = shift;
 my  %Info;  #array to collect info
 collectmarks () ; #run procedure to collect marks
 my $data="";
 my $IPRef = $self -> {'DESC'}{'ADV'};
 if (@$IPRef) {
 my $collect = \%collect_mark;
 $data= $collect->{'mstart'} ;
 my $packet = $self -> {'DESC'}{'TYPE'} ;     #0 bytes or 1 packets
 my @temptab= (0,0); #tempolary variable
 for (my $i=0; $i<@$IPRef; $i++ ) {
#  print $marked[1],"\n";
  my ($bytes, $pkts, $ref);
  $bytes =0;
  $pkts =0;
  my $MarkExp = $$IPRef[$i]->{'MARK'};
  while ($MarkExp =~ /([\+\-]?\w+)/g) {
  my $mark = $1;
  my $markname = $1;
  $markname =~ s/[\+\-]//g;
  if (defined ($collect-> {$markname})) {
   $ref =$collect-> {$markname};
  }  else {
#    &errorproc ("Brak zaznaczenia ip: $mark w $marked[3]\n");
   $ref = \@temptab;
  }
  if ($mark=~/^\-/) {
  #substarction
  $bytes -= $$ref[1];
  $pkts -= $$ref[0];
  } else {
  $bytes += $$ref[1];
  $pkts += $$ref[0];
  }
 } #end while


if ($packet) {
  $data  .= ":".$pkts;
 } else {
  $data  .= ":".$bytes;
 }

  } #ernd for
} #end if
 return ($data);
}


sub Draw_Object {
 my $self = shift;
 my $rrd =shift;
 my $Refsource =shift;

 my $curcolour=0;

 my $RefIP  = $self -> {'DESC'}{'ADV'}; #reference to statistic info
 my $Type = $self -> {'DESC'}{'TYPE'};     #0 bytes or 1 packets
 my $Bits = $self -> {'DESC'}{'BITS'};     #0 bytes/s or 1 bits per s
 my $Bar = $self -> {'DESC'}{'BAR'};     #0 line or 1 bar



  push (@$Refsource,"--vertical-label");
if ($Type) {
  push (@$Refsource,$dict{IP_VLABEL_PAC});
 } else {
 if ($Bits) {  push (@$Refsource,$dict{IP_VLABEL_BIT}) }
  else {
   push (@$Refsource,$dict{IP_VLABEL_BYTE});
  }
 }

#tempolary
my $longinfo=$self -> {'DESC'}{'EXTRA_INFO'};
my $refdata;

if (@$RefIP) {
 if ($longinfo) {
  #additonal info
   #eval some statistic
  $refdata = $self->ExtraData ();
  push (@$Refsource,"COMMENT:$dict{'IP_TABLE'}\\s");
  push (@$Refsource,"COMMENT:\\s");
if ($Type) {
  #packet
  push (@$Refsource,"COMMENT:                 [pkt]  [p/s]  [p/s]  [p/s]  [p/s]\\s");
} else
 { #bytes
   push (@$Refsource,"COMMENT:                  [B]  [B/s]  [B/s]  [B/s]  [B/s]\\s");
 }
  push (@$Refsource,"COMMENT:\\s");

 }
 my $spacing;
 my $labelsize=8;
 for (my $i=0; $i < @$RefIP; $i++) {
  push (@$Refsource,"DEF:m$i=$rrd:m$i:AVERAGE");
  if (!$Type && $Bits ) {
   push (@$Refsource,"CDEF:sm$i=m$i,8,*"); #for bits/s
   } else { push (@$Refsource,"CDEF:sm$i=m$i") }; #for bytes per s and packets
   my $Row = $$RefIP[$i]; #reference to hash
   my $colour = $Row->{'COLOR'};
   $colour = $defcolour [$curcolour++] if (! $colour); #automatic
   my $desc = $Row->{'DESC'};
   $desc =$Row->{'MARK'} if (! $desc);
  if (!$Bar) {
   #as linie
   push (@$Refsource,"LINE2:sm$i\#$colour:$desc");
  }  else {
   if ($i) {
    #not first bar
     push (@$Refsource,"STACK:sm$i\#$colour:$desc");
   } else {
   #first
    push (@$Refsource,"AREA:sm$i\#$colour:$desc");
   }
   }
 if ($longinfo) {
  #additonal info
  my $data = $refdata -> {"m$i"}; #stats for output
  $spacing = " " x ($labelsize -length ($desc));
  push (@$Refsource,"COMMENT: $spacing");
  #convert to Kilo Mega or Giga
  for (my $i=0; $i<@$data;$i++) { $data->[$i] = ConvB ($data->[$i])}

  push (@$Refsource,"COMMENT:$data->[0] $data->[1] $data->[3] $data->[2] $data->[4]");

  push (@$Refsource,"COMMENT:\\s");
  push (@$Refsource,"COMMENT:\\s");

 } #end add info
 } #end for
 }
}
#---------END TIPGRAPHER-----------------------------------

package TPINGGrapher;

use IPC::Open3;
use vars qw ( @ISA );
use Misc;
use Misc qw ( @defcolour );
@ISA = qw ( TGrapher );

sub new {
 my $proto = shift;
 my $class = ref ($proto) || $proto;
 my $self = $class -> SUPER::new ();
 LoadLangData ('','ping');
 #setting default values
 $self -> name ( 'pingXX');
 $self -> {'TITLE_TAG'} = 'PING_DEFTITLE';
 my @IP;
 $self -> {'DESC'}{'ADV'} = [ @IP ]; #reference to statistic info
 $self -> {'DESC'}{'BAR'} = 1;     #0 line or 1 bar
 $self -> {'DESC'}{'COUNT'} = 3;     #count ping packet
 $self -> {'DESC'}{'DELAY'} = 1000;  #delay beetwen packet
 $self -> {'DESC'}{'LOST'} = 0;  #display delay or lost packets
 $self->AddToRADIOBOX ('BAR','LOST'); #for editing as radio box
 $self -> {'HELP_FILE'} = 'ping.html'; #set help file

 bless ($self, $class);
 return $self;
}

sub Create_Object {
 my $self = shift;
 my $rrdstep = shift;
 my $Refsource = shift;

 my $PartRef =  $self -> {'DESC'}{'ADV'}; #reference to array of partitions


if (@$PartRef) { #has any elements
#   push (@$Refsource, "RRA:LAST:0.5:1:600");
#   push (@$Refsource, "RRA:LAST:0.5:6:700");
#   push (@$Refsource, "RRA:LAST:0.5:24:775");
#   push (@$Refsource, "RRA:LAST:0.5:288:797");
 for (my $i=0; $i < @$PartRef; $i++) {
   push (@$Refsource,"DS:avg$i:GAUGE:$rrdstep:0:U");
#   push (@$Refsource,"DS:lost$i:ABSOLUTE:$rrdstep:0:U");
   push (@$Refsource,"DS:lost$i:GAUGE:$rrdstep:0:U");

 }
 return 1;
 } else {return 0}

}

#edit additional primary fields
sub EditObjPrimary {
my $self = shift;
my $RefTags = shift;

#show additional template for this object
Misc::Gen ($RefTags, 'epingpri.html',1);
}


#prepare to edit partitions info
sub EditObjAdvanced {
 my $self = shift;
 my $RefTags = shift;

 my $PartRef =  $self -> {'DESC'}{'ADV'}; #reference to array of partitions

 Gen ($RefTags, 'ebegadv.html', 1);
 if (@$PartRef) { #has any element;
  print "<TABLE BORDER\=1 BGCOLOR=\"#CACAFF\" CELLSPACING=0 CELLPADDING=0 >\n";
  print "<TR BGCOLOR=\"#A0A0FF\"><B><TD><FONT COLOR=red>*</FONT>$dict{PING_EDIT_POS}</TD><TD><FONT COLOR=red>*</FONT>\n";
  print "$dict{PING_EDIT_ADDR}</TD><TD>$dict{PING_EDIT_COLOR}</TD><TD>$dict{PING_EDIT_DESC}</TD></B></TR>\n";
  for (my $i=0; $i<@$PartRef; $i++) {
   my $HashRef = $$PartRef [$i];
   print "<TR><TD><INPUT TYPE=CHECKBOX NAME=\"$i\" ></TD>\n";
   print "<TD><INPUT TYPE=\"text\" NAME=\"ADDR$i\" VALUE=\"";
   print $$HashRef {'ADDR'};
   print "\" SIZE=20 MAXLENGTH=40></TD>\n";

   print "<TD><INPUT TYPE=\"text\" NAME=\"COLOR$i\" VALUE=\"";
   print $$HashRef {'COLOR'};
   print "\" SIZE=8 MAXLENGTH=8></TD>\n";

   print "<TD><INPUT TYPE=\"text\" NAME=\"DESC$i\" VALUE=\"";
   print $$HashRef {'DESC'};
   print "\" SIZE=20 MAXLENGTH=60></TD>\n";

   print "</TR>\n";
  }
  print "</TABLE>\n";
 }
 Gen ($RefTags, 'eendadv.html', 1);
}

sub PostEditObjAdvanced {
 my $self = shift;
 my $RefPostTags =shift;

 my $Result=0;
 #catch end form's action
 $Result = $RefPostTags->{'RESULT'} if ( defined ($RefPostTags->{'RESULT'}));

 if (( $Result eq 'NEW_EL') or #run only if advanced data was edited
     ( $Result eq 'DEL_EL') or
     ( $Result eq 'OK_EL') ) {
 my @NewPart;
 $self -> {'DESC'}{'ADV'} = [ @NewPart ]; #reference to array of partitions
 my $PartRef = $self -> {'DESC'}{'ADV'};
 #save all elements to array
 my $i = 0;
 my $del = ($Result eq 'DEL_EL'); #if delete any element
 while (defined ($RefPostTags->{"ADDR$i"}) ) {
  if (! (defined ($RefPostTags->{$i}) && $del )) {
  my %row;   #add to data
  $row {'ADDR'} = $RefPostTags->{"ADDR$i"};
  $row {'COLOR'} = $RefPostTags->{"COLOR$i"};
  $row {'DESC'} = $RefPostTags->{"DESC$i"};
  push (@$PartRef, { %row } );
  }
  $i++;
 }

 if ($Result eq 'NEW_EL') {
  #pressed New Element
  my %NewHash;
  $NewHash {'ADDR'} = "";
  $NewHash {'COLOR'} = "";
  $NewHash {'DESC'} = "";
  push (@$PartRef, { %NewHash });
 }
}
}

#check if need recreate RRD file
sub CompareEditObj {
 my $self = shift;
 my $newobj = shift;
 my $Result = 0;
 my $newobjpart = $newobj-> {'DESC'}{'ADV'};
 my $objpart = $self-> {'DESC'}{'ADV'};
 if ($#$newobjpart != $#$objpart ) {
  $Result=1; #difference in elements count
  }
return ($Result);
}


#check security
sub Check_Security {
 my $self = shift;
 my $ok=0;
 my $count = $self -> {'DESC'}{'COUNT'} ;     #count ping packet
 my $delay = $self -> {'DESC'}{'DELAY'};  #max delay beetwen packet
 if ( $count and $delay) {
 if ((length ($count) < 3) and (length ($delay) < 6)) {
 if (($count =~ /\D/) or ($delay =~ /\D/)) {
  return ($ok); #insecure
 } else {
  $ok=1; #object OK
 }
 }
 }
 return ($ok);
}

#update values in RRD file
sub Update_Object {
 my $self = shift;
 my $data=0;
 my $IPRef = $self -> {'DESC'}{'ADV'};
 my $count = $self -> {'DESC'}{'COUNT'} ;     #count ping packet
 my $delay = $self -> {'DESC'}{'DELAY'} = 1000;  #max delay beetwen packet

 if (@$IPRef) {
 my $fping = $self -> {'CONFIG'}{'FPING'};
 if (defined ($fping)) {
 if ($fping) {
 my %OutArray;

 my $fpingargs = "-C $count -p $delay -A -q -u";
 open3(\*INPUT, \*OUTPUT, \*ERROR, $fping, $fpingargs);

 foreach my $RefHash (@$IPRef) {
  print INPUT "$$RefHash{'ADDR'}\n";
  }
 close(INPUT);
 my @output=<OUTPUT>;
 my @erroutput=<ERROR>;

 foreach my $line (@erroutput) {
  if ($line =~ /:/) {
   chomp ($line);
   my ($comp, $data) = split (/:/,$line,2);
   $comp =~ s/\s+//g;
   $data =~ s/^\s+//g;
   $data =~ s/\s+$//g;
   my @results = split (/\s+/,$data);

   my $sum=0;
   my $lost=0;
   my $okpacket=0;
   foreach my $res (@results) {
    if (! ($res =~ /-/)) {
     $sum += $res;
     $okpacket++;
    } else { $lost++;}

   }
   my $avg=0;
   $avg=$sum/$okpacket    if ($okpacket);
   # my $kas=/"/";
   my %ThisResult;
   $ThisResult {'AVG'} = $avg;
   $ThisResult {'LOST'} = $lost;
   #add to outarray
   $OutArray {$comp} = { %ThisResult };
  }
 }
 close (OUTPUT);
 close (ERROR);

 #store time
 $data=time;

 #find results
 foreach my $RefHash (@$IPRef) {
  my $addr = $$RefHash{'ADDR'};
  my $avg='U';
  my $lost='U';
  if (defined ($OutArray {$addr})) {
   my $ref = $OutArray {$addr};
   $avg = $$ref {'AVG'};
   $lost = $$ref {'LOST'};
  }
  $data .="\:$avg\:$lost";
  }
}}
} #end if
 return ($data);
}


sub Draw_Object {
 my $self = shift;
 my $rrd =shift;
 my $Refsource =shift;

 my $curcolour=0;

 my $RefIP  = $self -> {'DESC'}{'ADV'}; #reference to statistic info
 my $Bar = $self -> {'DESC'}{'BAR'};     #0 line or 1 bar
 my $lost = $self -> {'DESC'}{'LOST'};   #dispaly lost packets


 push (@$Refsource,"--vertical-label");
 if ($lost) {
  push (@$Refsource,$dict{IP_VLABEL_LOST});
 } else
 {
  push (@$Refsource,$dict{IP_VLABEL_RESPONSE});
 }

if (@$RefIP) {
 for (my $i=0; $i < @$RefIP; $i++) {
  if ($lost) {
  push (@$Refsource,"DEF:avg$i=$rrd:lost$i:AVERAGE");
  } else
  {
  push (@$Refsource,"DEF:avg$i=$rrd:avg$i:AVERAGE");
  }
   my $Row = $$RefIP[$i]; #reference to hash
   my $colour = $Row->{'COLOR'};
   $colour = $defcolour [$curcolour++] if (! $colour); #automatic
   my $desc = $Row->{'DESC'};
   $desc =$Row->{'ADDR'} if (! $desc);
  if (!$Bar) {
   #as linie
   push (@$Refsource,"LINE2:avg$i\#$colour:$desc");
  }  else {
   if ($i) {
    #not first bar
     push (@$Refsource,"STACK:avg$i\#$colour:$desc");
   } else {
   #first
    push (@$Refsource,"AREA:avg$i\#$colour:$desc");
   }
   }

 } #end for
 } #end if
}

#-------end TPINGGrapher

